#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2009 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>
#
# NOTE:
# This file belongs in /usr/lib/nautilus/extensions-2.0/python/

print "Initializing Wine integration: Bottle selection"

import urllib
import os, sys, subprocess
import threading, time
import wine

import gtk, gobject, pango
import nautilus
import gettext as _gettext
from locale import getdefaultlocale

SUPPORTED_FORMATS = 'application/x-ms-dos-executable','application/x-msdos-program','application/x-msdownload','application/exe','application/x-exe','application/dos-exe','vms/exe','application/x-winexe','application/msdos-windows','application/x-zip-compressed','application/x-executable','application/x-msi'

SHARED_FILES_PATH = None
if os.path.isdir( "%s/locale" % os.path.abspath(os.path.dirname(sys.argv[0])) ):
	SHARED_FILES_PATH = os.path.abspath(os.path.dirname(sys.argv[0]))
else:
	for path in [ os.path.sep.join(i.split(os.path.sep)[:-1]) for i in os.environ['PATH'].split(':') ]:
		if os.path.isdir( "%s/share/vineyard" % path):
			SHARED_FILES_PATH = "%s/share/vineyard" % path
		
if SHARED_FILES_PATH == None:
	print "Something is wrong with the installation, can't find required files. Exiting."
	exit(1)

APP_NAME = "vineyard"

languages = ( [getdefaultlocale()[0]] or [] )
if 'LANGUAGE' in os.environ:
	languages = os.environ['LANGUAGE'].split(':') + languages
elif 'LANG' in os.environ:
	languages = os.environ['LANG'].split(':') + languages

_gettext.bindtextdomain(APP_NAME, "%s/%s" % (SHARED_FILES_PATH, "locale"))
_gettext.textdomain(APP_NAME)

gettext = _gettext.translation(APP_NAME, "%s/%s" % (SHARED_FILES_PATH, "locale"), languages=languages, fallback=True)
_ = gettext.gettext


#gobject.threads_init()
#gtk.gdk.threads_init()
#gtk.gdk.threads_enter()

class BottleDialog(gtk.Dialog):
	def __init__(self, parent, file):
		super(BottleDialog, self).__init__(title=_("Run in Wine configuration"), parent=parent)
		
		self.parentwidget = parent
		self.file = file
		
		self.hbox = gtk.HBox()
		self.hbox.set_spacing(6)
		self.hbox.set_border_width(6)
		self.innervbox = gtk.VBox()
		self.innervbox.set_spacing(6)
		self.hbox.pack_end(self.innervbox)
		self.vbox.add(self.hbox)
		
		self.image = gtk.Image()
		self.image.set_from_icon_name("wine", gtk.ICON_SIZE_DIALOG)
		self.image.set_alignment(0.5, 0.0)
		self.hbox.pack_start(self.image, expand=False, fill=False)
		
		self.liststore = gtk.ListStore(str)
		
		self.bottles = [_('Default')] + wine.bottles.list()
		
		self.liststore = gtk.ListStore(str)
		self.bottlewidget = gtk.ComboBox(self.liststore)
		cell = gtk.CellRendererText()
		self.bottlewidget.pack_start(cell, True)
		self.bottlewidget.add_attribute(cell, 'text', 0)
		
		for bottle in self.bottles:
			self.liststore.append([bottle])
		
		self.liststore.append(['-'])
		self.liststore.append([_("New configuration")])
		
		self.bottlewidget.set_row_separator_func(self.__row_separator_func)
		
		GtkSelectCombobox(self.bottlewidget, self.bottles[0])
		
		self.bottlewidget.connect("changed", self.__bottle_selection_changed)

		self.innervbox.pack_start(self.bottlewidget, False, False)
		
		self.newvbox = gtk.VBox()
		self.entryhbox = gtk.HBox()
		self.entryhbox.set_spacing(6)
		
		self.entrylabel = gtk.Label(_("Name: "))
		self.entryhbox.pack_start(self.entrylabel, False, False)
		
		self.entry = gtk.Entry()
		self.entrycompletion = gtk.EntryCompletion()
		self.entry.set_completion(self.entrycompletion)
		self.entrycompletion.set_model(self.liststore)
		self.entrycompletion.set_text_column(0)
		
		self.entry.connect("changed", self.__entry_changed)
		
		self.entryhbox.pack_start(self.entry)
		self.newvbox.pack_start(self.entryhbox, False, False)
		
		self.expander = gtk.Expander(_("New configuration options"))
		
		self.table = gtk.Table(rows=2, columns=2, homogeneous=False)

		version_label = gtk.Label(_('Operate as:'))
		version_label.set_alignment(0.0, 0.5)
		self.version_value = gtk.combo_box_new_text()
		for version in ['Windows 7', 'Windows Vista', 'Windows 2003', 'Windows XP %s' % _("(default)"), 'Windows 2000', 'Windows NT 4.0', 'Windows NT 3.51', 'Windows ME', 'Windows 98', 'Windows 95', 'Windows 3.11', 'Windows 3.0', 'Windows 2.0']:
			self.version_value.append_text(version)
		GtkSelectCombobox(self.version_value, "Windows XP", startswith=True)
		self.table.attach(version_label, 0,1, 0,1, gtk.FILL,0, 6,6)
		self.table.attach(self.version_value, 1,2, 0,1, gtk.FILL,0, 6,6)

		desktop_label = gtk.Label(_('Open in:'))
		desktop_label.set_alignment(0.0, 0.0)
		desktop_value_box = gtk.VBox()
		self.desktop_value_check = gtk.CheckButton(_('Open program windows in a virtual desktop'))
		desktop_value_box.pack_start(self.desktop_value_check)
		self.desktop_table = gtk.Table(rows=2, columns=3, homogeneous=False)
		desktop_width = gtk.Label(_('Desktop width: '))
		desktop_width.set_padding(24,0)
		desktop_width.set_alignment(0.0, 0.5)
		self.desktop_width_spin = gtk.SpinButton(climb_rate=1.0)
		self.desktop_width_spin.get_adjustment().set_all(1024.0, lower=0, upper=10000, step_increment=1, page_increment=10, page_size=0)
		desktop_width_label = gtk.Label(_('pixels'))
		desktop_width_label.set_padding(6,0)
		desktop_height = gtk.Label(_('Desktop height: '))
		desktop_height.set_padding(24,0)
		desktop_height.set_alignment(0.0, 0.5)
		self.desktop_height_spin = gtk.SpinButton(climb_rate=1.0)
		self.desktop_height_spin.get_adjustment().set_all(768.0, lower=0, upper=10000, step_increment=1, page_increment=10, page_size=0)
		desktop_height_label = gtk.Label(_('pixels'))
		desktop_height_label.set_padding(6,0)
		self.desktop_table.attach(desktop_width, 0,1, 0,1, gtk.FILL,0, 0,0)
		self.desktop_table.attach(self.desktop_width_spin, 1,2, 0,1, gtk.FILL,0, 0,0)
		self.desktop_table.attach(desktop_width_label, 2,3, 0,1, gtk.FILL,0, 0,0)
		self.desktop_table.attach(desktop_height, 0,1, 1,2, gtk.FILL,0, 0,0)
		self.desktop_table.attach(self.desktop_height_spin, 1,2, 1,2, gtk.FILL,0, 0,0)
		self.desktop_table.attach(desktop_height_label, 2,3, 1,2, gtk.FILL,0, 0,0)
		self.desktop_table.set_sensitive(False)
		desktop_value_box.pack_start(self.desktop_table)
		self.table.attach(desktop_label, 0,1, 1,2, gtk.FILL,gtk.FILL, 6,6)
		self.table.attach(desktop_value_box, 1,2, 1,2, gtk.FILL,0, 6,6)

		self.expander.add(self.table)
		self.newvbox.set_sensitive(False)
		self.newvbox.pack_start(self.expander, False, False)
		self.innervbox.pack_start(self.newvbox, False, False)
		
		self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		self.add_button(gtk.STOCK_OK, gtk.RESPONSE_YES)

		self.set_default_response(gtk.RESPONSE_YES)

		self.show_all()
		
		self.desktop_value_check.connect("toggled", self.__set_desktop)
        
		self.entry.connect("key-press-event", self.__key_pressed)
		
		result = self.run()
		
		if result == gtk.RESPONSE_YES:
			self.__ok()
		else:
			self.destroy()
	
	def __row_separator_func(self, model, iter):
		return model.get_value(iter,0) == '-'
	
	def __set_desktop(self, checkbox):
		if checkbox.get_active():
			self.desktop_table.set_sensitive(True)
		else:
			self.desktop_table.set_sensitive(False)
	
	def __bottle_selection_changed(self, combobox):
		selected = combobox.get_model()[combobox.get_active()][0]
		if selected == combobox.get_model()[-1][0]:
			self.newvbox.set_sensitive(True)
		else:
			self.newvbox.set_sensitive(False)
	
	def __entry_changed(self, entry):
		value = entry.get_text()
		
		if not len(value.strip()):
			return
		
		select = None
		for rownr in range(len(self.bottles)):
			if self.bottles[rownr].lower() == value.lower().strip():
				select = rownr
				break
		if select:
			self.bottlewidget.set_active(select)
			style = self.entry.get_style()
			style.font_desc.set_style(pango.STYLE_ITALIC)
			self.entry.set_style(style)
			print style.font_desc
		else:
			style = self.entry.get_style() 
			style.font_desc.set_style(pango.STYLE_NORMAL)
			self.entry.set_style(style)
	
	def __ok(self):
		self.hide()
		selectedbottle = self.liststore[self.bottlewidget.get_active()][0]
		if selectedbottle == self.liststore[-1][0]:
			bottlename = self.entry.get_text().strip()
			selectedbottle = bottlename
			self.__create_bottle(bottlename)
		elif selectedbottle == self.liststore[0][0]:
			selectedbottle = None
		self.__run_file(self.file, selectedbottle)
		
	def __key_pressed(self, widget, event):
		# If Enter was pressed
		if event.keyval == 65293:
			self.response(gtk.RESPONSE_YES)
	
	def __create_bottle(self, bottlename):
		self.creatingbottledialog = gtk.Dialog('',
		                                       None,
		                                       gtk.DIALOG_DESTROY_WITH_PARENT,
		                                       None)
		self.creatingbottledialog.set_has_separator(False)
		vbox = gtk.VBox()
		self.creatingbottledialog.vbox.add(vbox)
		vbox.set_spacing(6)
		vbox.set_border_width(6)
		self.creatingbottlelabel = gtk.Label(_("Creating configuration \"%s\"") % bottlename)
		vbox.pack_start(self.creatingbottlelabel, False, False)
		self.creatingbottleprogress = gtk.ProgressBar()
		self.creatingbottleprogress.set_pulse_step(0.1)
		vbox.add(self.creatingbottleprogress)
		#self.creatingbottledialog.action_area.get_children()[0].connect("activate", self.__create_bottle_cancel)
		self.creatingbottledialog.show_all()
		self.bottlename = bottlename
		self.creatingbottledone = False
		gobject.timeout_add(60, self.__create_bottle_interface)
		start_thread(self.__create_bottle_backend)
	
	def __create_bottle_backend(self):
		wine.bottles.add(self.bottlename)
		self.__create_bottle_backend_version()
		self.__create_bottle_backend_desktop()
		self.creatingbottledone = True
	
	def __create_bottle_backend_version(self):
		version = "winxp"
		selected = self.version_value.get_model()[self.version_value.get_active()][0]
		# Fix the special case of Windows 7 being called Windows 2008 in Wine
		if selected == 'Windows 7':
			selected = 'Windows 2008'
		# Remove any " (default)" from the name
		selected = selected.split(' (')[0]
		# Convert the name (f.x. Windows 2000) to the version number (f.x. win2k)
		for i in [ (key, value[0]) for key,value in wine.version.windowsversions.iteritems() ]:
			if i[1] == selected:
				version = i[0]
		wine.version.set(version)
	
	def __create_bottle_backend_desktop(self):
		if self.desktop_value_check.get_active():
			wine.desktop.set(True, size = (
				self.desktop_width_spin.get_value_as_int(),
				self.desktop_height_spin.get_value_as_int()
			))
		else:
			wine.desktop.set(False)
	
	def __create_bottle_interface(self):
		if self.creatingbottledone == True:
			self.creatingbottledialog.destroy()
			return False
		elif self.creatingbottledone == False:
			self.creatingbottleprogress.pulse()
			return True
	
	def __run_file(self, filename, bottlename):
		wine.bottles.use(bottlename)
		wine.run(filename)

class SelectBottle(nautilus.MenuProvider):
	def __init__(self):
		pass
	
	def __select_bottle(self, filename):
		dialog = BottleDialog(parent=self.window, file=filename)
	
	def menu_activate(self, menu, file):
		if file.is_gone():
			return

		self.__select_bottle(urllib.unquote(file.get_uri()[7:]))

	def get_file_items(self, window, files):
		if len(files) != 1:
			return
		
		file = files[0]
		if file.is_directory() or file.get_uri_scheme() != 'file' or file.get_mime_type() not in SUPPORTED_FORMATS:
			return
		
		self.window = window
		
		item = nautilus.MenuItem('NautilusPython::vineyard_configuration_selection',
		                         _("Run in Wine configuration") ,
		                         _("Run %s in a new or existing Wine configuration/bottle") % file.get_name(),
		                         'wine')
		item.connect('activate', self.menu_activate, file)
		return item,

def start_thread(function):
	thread = threading.Thread(target=function)
	thread.start()
	while thread.isAlive():
		while gtk.events_pending():
			gtk.main_iteration()

def GtkSelectCombobox(combobox, string, startswith=False):
	string = string.lower()
	model = combobox.get_model()
	for rownr in range(len(model)):
		if (startswith and model[rownr][0].lower().startswith(string)) or (not startswith and model[rownr][0].lower() == string):
			combobox.set_active(rownr)
			break

if __name__ == "__main__":
	try:
		main = BottleDialog(None, sys.argv[1])
	finally:
		pass

#gtk.gdk.threads_leave()
