#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2009 Christian Dannie Storgaard
#
import gobject, gtk, pango, re, os
import common, icons
import wine

_PARENT_WINDOW = None
RUNNING_PROGRAMS = {}

class Page:
    def __init__(self, parent_window=None):
        _PARENT_WINDOW = parent_window
        # FIXME: The following info is not yet used
        self.icon = 'package-x-generic'
        self.name = _("Programs")
        self.title = _("Installed Programs")
        self.id = 'programs'
        self.position = 0.9
        self.settings = {'programs': {}, 'menuprograms': {}, 'selectedprogram': None}
        self._build_interface()
        self.program_editors = {}
    
    def _build_interface(self):
        self.widgets = {}
        self.widgets['vbox'] = gtk.VBox(spacing=6)
        self.widget = self.widgets['vbox']
        
        self.widgets['scrolled_window'] = gtk.ScrolledWindow()
        self.widgets['scrolled_window'].set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.widgets['vbox'].pack_start(self.widgets['scrolled_window'], expand=True, fill=True)
        self.widgets['treeview'] = gtk.TreeView()
        self.widgets['treeview'].set_headers_visible(False)
        self.widgets['treeview'].set_headers_clickable(False)
        self.widgets['scrolled_window'].add(self.widgets['treeview'])

        self.widgets['treeview_column'] = gtk.TreeViewColumn(_("Application"))
        self.widgets['treeview_column'].set_expand(True)
        self.widgets['treeview_column'].set_sort_column_id(1)
        
        renderer_app_icon = gtk.CellRendererPixbuf()
        self.widgets['treeview_column'].pack_start(renderer_app_icon, False)
        self.widgets['treeview_column'].set_cell_data_func(renderer_app_icon, common.cell_data_function_icon_markup_icon)
        
        #renderer_app = gtk.CellRendererText()
        renderer_app = common.CellRendererHeaderText()
        renderer_app.set_property("ellipsize", pango.ELLIPSIZE_END)
        self.widgets['treeview_column'].pack_start(renderer_app, True)
        #self.widgets['treeview_column'].set_cell_data_func(renderer_app, common.cell_data_function_icon_markup_markup)
        self.widgets['treeview_column'].add_attribute(renderer_app, 'markup', 1)
        
        self.widgets['treeview'].append_column(self.widgets['treeview_column'])
        
        self.widgets['treeview']._drag_start_position = None
        
        self.widgets['treeview'].connect('cursor-changed', self._on_treeview_cursor_changed)
        self.widgets['treeview'].connect('button-press-event', self._on_treeview_button_press_event)
        self.widgets['treeview'].connect('button-release-event', self._on_treeview_button_release_event)
        self.widgets['treeview'].connect('drag-begin', self._on_treeview_drag_begin)
        self.widgets['treeview'].connect('drag-data-get', self._on_treeview_drag_data_get)
        self.widgets['treeview'].connect('motion-notify-event', self._on_treeview_motion_notify_event)
        
        self.widgets['table'] = gtk.Table(rows=2, columns=2)
        self.widgets['table'].set_col_spacings(6)
        self.widgets['table'].set_row_spacings(6)
        self.widgets['vbox'].pack_start(self.widgets['table'], expand=False, fill=True)
        
        self.widgets['button_run'] = common.button_new_with_image('gtk-execute', _("_Run"))
        self.widgets['button_run'].connect('clicked', self._on_button_run_clicked)
        self.widgets['table'].attach(self.widgets['button_run'], 0,1, 0,1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        
        self.widgets['button_uninstall'] = common.button_new_with_image('gtk-remove', _("_Uninstall"))
        self.widgets['button_uninstall'].connect('clicked', self._on_button_uninstall_clicked)
        self.widgets['table'].attach(self.widgets['button_uninstall'], 1,2, 0,1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        
        self.widgets['button_add'] = common.button_new_with_image('gtk-add', _("A_dd unlisted program"))
        self.widgets['button_add'].connect('clicked', self._on_button_add_clicked)
        self.widgets['table'].attach(self.widgets['button_add'], 0,2, 1,2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)

        self.widgets['menu_programs'] = gtk.Menu()
        self.widgets['menu_programs_edit'] = gtk.MenuItem(_("_Edit program"), True)
        self.widgets['menu_programs_edit'].connect("activate", self._on_program_edit)
        self.widgets['menu_programs'].append(self.widgets['menu_programs_edit'])
        self.widgets['menu_programs_refresh'] = gtk.MenuItem(_("_Refresh program list"), True)
        self.widgets['menu_programs_refresh'].connect("activate", self._on_programs_refresh)
        self.widgets['menu_programs'].append(self.widgets['menu_programs_refresh'])
        self.widgets['menu_programs'].show_all()
        
        self.widgets['vbox'].show_all()
    
    def _get_markup_from_program_data(self, program_data):
        if 'description' in program_data:
            return "%s\n<small>%s</small>" % (program_data['name'], program_data['description'])
        else:
            return program_data['name']
    
    def _model_add_section(self, model, header, programdict, first=False):
        if first == True:
            model.append([None, '<header first>%s</header>' % header])
        else:
            model.append([None, '<header>%s</header>' % header])
        for program_id in sorted(programdict.keys(), key=unicode.lower):
            program_data = programdict[program_id]
            
            model.append([
                icons.getIconPixbufFromProgram(program_data),
                self._get_markup_from_program_data(program_data)
            ])
    
    def reload(self, settings):
        self.widgets['treeview'].set_sensitive(False)
        
        self.settings.update(settings.dict)
        self.settings['programs'] = settings['programs']
        self.settings['menuprograms'] = settings['menuprograms']
        
        liststore = gtk.ListStore(gtk.gdk.Pixbuf, str)
        
        #print "Building treeview with the programs:", programs
        registered_programs = dict(
            (unicode(k), v) for (k, v) in settings['programs'].iteritems()
            if 'programexe' in v or 'programcommand' in v
        )
        registered_libraries = dict(
            (unicode(k), v) for (k, v) in settings['programs'].iteritems()
            if 'programexe' not in v and 'programcommand' not in v
        )
        weblinks = {}
        menu_programs = {}
        # Seperate real programs from links to websites
        for key,value in settings['menuprograms'].iteritems():
            if 'programcommand' in value and re.search(r'(?i).*\.(url)', value['programcommand']):
                value['website'] = True
                weblinks[unicode(key)] = value
            else:
                menu_programs[unicode(key)] = value

        for program in menu_programs:
            if 'path' in program:
                program['name'] = u"%s: %s" % (program['path'], program['name'])

        self._model_add_section(liststore, _('Registered Programs'), registered_programs, first=True)
        self._model_add_section(liststore, _('Registered Libraries'), registered_libraries)
        self._model_add_section(liststore, _('Menu Programs'), menu_programs)
        self._model_add_section(liststore, _('Websites'), weblinks)

        self.widgets['treeview'].set_model(liststore)
        self.widgets['treeview'].expand_all()
        #self.widgets['treeview-programs'].get_selection().select_path((1,0))
        if len(liststore):
            self.widgets['treeview'].set_cursor((1,))
        self.widgets['treeview'].set_sensitive(True)

    def _on_button_run_clicked(self, button):
        program = self.get_program_data_from_name(self.settings['selectedprogram'])

        if 'programcommand' in program:
            RunAndMonitorProgram(program['programcommand'], program['name'])
        elif 'programexe' in program:
            RunAndMonitorProgram(program['programexe'], program['name'])
        else:
            return False

    def _on_button_uninstall_clicked(self, button):
        wine.programs.uninstall(self.settings['selectedprogram'].lower())
    
    """
        Program editing and adding functions
    """
    
    def _on_button_add_clicked(self, button):
        program_data = {
            '_registrykey': "vineyard-program-%s" % wine.util.tempstring(),
            'name': '',
            'description': '',
            'terminal': False,
            'uninstall': ''
        }
        self.program_editors['new_program'] = ProgramEditor('new_program', program_data, self)

    
    """
        Signal Handlers
        ---------------
    """
    
    """ Treeview signal handlers """
    
    #def onPrograms(self, treeview):
    def _on_treeview_cursor_changed(self, treeview):
        selected = unicode(common.treeview_get_selected_row(treeview)[1].split('\n')[0])
        
        """ If the user clicked a header, select the first non-header row instead """
        if selected.startswith('<header'):
            model = treeview.get_model()
            for rownr in range(len(model)):
                if not model[rownr][1].split('\n')[0].startswith('<header'):
                    treeview.set_cursor((rownr,))
                    return False
            treeview.get_selection().unselect_all()
            return False

        self.settings['selectedprogram'] = selected

        program_data = self.get_program_data_from_name(selected)
        
        if 'programexe' in program_data or 'programcommand' in program_data:
            self.widgets['button_run'].set_sensitive(True)
        else:
            self.widgets['button_run'].set_sensitive(False)
        if 'uninstall' in program_data:
            self.widgets['button_uninstall'].set_sensitive(True)
        else:
            self.widgets['button_uninstall'].set_sensitive(False)

    def _on_treeview_button_press_event(self, widget, event):
        if event.type != gtk.gdk.BUTTON_PRESS:
            return False

        # DND handling
        if event.button == 1:
            widget._drag_start_position = (int(event.x), int(event.y))
        # Menu handling
        elif event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            clickedpath = self.widgets['treeview'].get_path_at_pos(x, y)
            if clickedpath != None:
                path, col, cellx, celly = clickedpath
                clickedrow = self.widgets['treeview'].get_model()[path]
                if not clickedrow[1].startswith('<header>'):
                    self.widgets['treeview'].grab_focus()
                    self.widgets['treeview'].set_cursor( path, col, 0)
                    program_data = self.get_program_data_from_name(self.settings['selectedprogram'])
                    if 'website' in program_data and program_data['website']:
                        self.widgets['menu_programs_edit'].hide()
                    else:
                        self.widgets['menu_programs_edit'].show()
                    self.widgets['menu_programs'].popup( None, None, None, event.button, time)
            return True
        return False

    def _on_treeview_button_release_event(self, widget, event):
        # DND Handling
        if event.button == 1:
            widget._drag_start_position = None
        return False

    def _on_treeview_drag_begin(self, widget, context):
        context.set_icon_pixbuf(widget._drag_pixbuf, 0, 0)
        return False

    def _on_treeview_drag_data_get(self, widget, context, selection, targetType, eventTime):
        if targetType == 1:
            program = self.get_program_data_from_name(self.settings['selectedprogram'])
            if program:
                desktopfile = wine.programs.createDesktopFile(program)
                if desktopfile:
                    selection.set(selection.target, 8, "file://%s" % desktopfile)

    def _on_treeview_motion_notify_event(self, widget, event):
        # DND Handling
        if widget._drag_start_position != None:
            startx, starty = widget._drag_start_position
            if widget.drag_check_threshold(startx, starty, int(event.x), int(event.y)):
                widget._drag_start_position = None
                clickedpath = self.widgets['treeview'].get_path_at_pos(int(event.x), int(event.y))[0]
                if clickedpath != None:
                    clickedrow = self.widgets['treeview'].get_model()[clickedpath]
                    if not clickedrow[1].startswith('<header>'):
                        # Check if there is any point in creating a launcher for this program (can it execute at all?)
                        selectedrow = common.treeview_get_selected_row(self.widgets['treeview'])
                        widget._drag_pixbuf = selectedrow[0]
                        self.settings['selectedprogram'] = selectedrow[1].split('\n')[0]
                        program = self.get_program_data_from_name(self.settings['selectedprogram'])
                        if program and 'programexe' in program or 'programcommand' in program:
                            widget.drag_begin([("text/uri-list", 0, 1)], gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE, 1, event)
        return True
    
    """ Right click menu signal handlers """
    
    def _on_program_edit(self, menuitem):
        program_id = self.settings['selectedprogram']
        program_data = self.get_program_data_from_name(program_id)
        self.program_editors[program_id] = ProgramEditor(program_id, program_data, self)
    
    def _on_programs_refresh(self, menuitem=None):
        self.widgets['treeview'].set_sensitive(False)
        self.__cache.clear()
        start_thread(self.get_settings)
    
    """ Utility functions """
    
    def get_program_data_from_name(self, name):
        name = name.split('\n')[0]
        if name in self.settings['programs']:
            return self.settings['programs'][name]
        elif name in self.settings['menuprograms']:
            return self.settings['menuprograms'][name]
        else:
            self.debug("Something went wrong, a click was detected on a program I don't know: %s" % name)
            return None



class ProgramEditor:
    def __init__(self, program_id, program_data, parent_class):
        self.program_id = program_id
        self.program_data = program_data
        self.settings = parent_class.settings
        self.parent_class = parent_class
        self._build_interface()
        self._update_interface_values()
    
    def _build_interface(self):
        self.widgets = {}
        self.widgets['dialog'] = self.dialog = gtk.Dialog(_("Program Information"),
            parent = _PARENT_WINDOW,
            flags = gtk.DIALOG_DESTROY_WITH_PARENT,
            buttons = (
                gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                gtk.STOCK_OK, gtk.RESPONSE_OK
            ))
        self.widgets['dialog'].set_icon_name('vineyard-preferences')
        self.widgets['dialog'].set_border_width(5)
        self.widgets['hbox'] = gtk.HBox(spacing=6)
        self.widgets['dialog'].vbox.pack_start(self.widgets['hbox'], expand=True, fill=True)
        
        self.widgets['vbox_icon'] = gtk.VBox()
        self.widgets['hbox'].pack_start(self.widgets['vbox_icon'], expand=False, fill=False)
        self.widgets['button_icon'] = gtk.Button(label=None, stock=None, use_underline=False)
        self.widgets['vbox_icon'].pack_start(self.widgets['button_icon'], expand=False, fill=False)
        self.widgets['aspectframe_icon'] = gtk.AspectFrame(label=None, xalign=0.5, yalign=0.5, ratio=1.0, obey_child=True)
        self.widgets['aspectframe_icon'].set_shadow_type(gtk.SHADOW_NONE)
        self.widgets['button_icon'].add(self.widgets['aspectframe_icon'])
        self.widgets['image_icon'] = self.image = gtk.Image()
        self.widgets['image_icon'].set_from_icon_name('application-x-executable', gtk.ICON_SIZE_DIALOG)
        self.widgets['image_icon'].set_pixel_size(gtk.icon_size_lookup(gtk.ICON_SIZE_DIALOG)[1])
        self.widgets['aspectframe_icon'].add(self.widgets['image_icon'])
        
        self.widgets['table'] = gtk.Table(rows=5, columns=3)
        self.widgets['table'].set_row_spacings(6)
        self.widgets['table'].set_col_spacings(6)
        self.widgets['hbox'].pack_start(self.widgets['table'], expand=True, fill=True)
        self.widgets['label_type'] = gtk.Label('%s:' % _("_Type"))
        self.widgets['label_type'].set_use_underline(True)
        self.widgets['label_type'].set_alignment(1.0, 0.5)
        self.widgets['table'].attach(self.widgets['label_type'],
            0,1, 0,1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_name'] = gtk.Label('%s:' % _("_Name"))
        self.widgets['label_name'].set_use_underline(True)
        self.widgets['label_name'].set_alignment(1.0, 0.5)
        self.widgets['table'].attach(self.widgets['label_name'],
            0,1, 1,2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_command'] = gtk.Label('%s:' % _("Comm_and"))
        self.widgets['label_command'].set_use_underline(True)
        self.widgets['label_command'].set_alignment(1.0, 0.5)
        self.widgets['table'].attach(self.widgets['label_command'],
            0,1, 2,3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_description'] = gtk.Label('%s:' % _("_Description"))
        self.widgets['label_description'].set_use_underline(True)
        self.widgets['label_description'].set_alignment(1.0, 0.5)
        self.widgets['table'].attach(self.widgets['label_description'],
            0,1, 3,4, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_uninstall'] = gtk.Label('%s:' % _("_Uninstall Command"))
        self.widgets['label_uninstall'].set_use_underline(True)
        self.widgets['label_uninstall'].set_alignment(1.0, 0.5)
        self.widgets['table'].attach(self.widgets['label_uninstall'],
            0,1, 4,5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['combobox_type'] = gtk.combo_box_new_text()
        for row in [_("Application"), _("Application in Terminal")]:
            self.widgets['combobox_type'].append_text(row)
        self.widgets['combobox_type'].set_active(0)
        self.widgets['table'].attach(self.widgets['combobox_type'],
            1,3, 0,1, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_type'].set_mnemonic_widget(self.widgets['combobox_type'])
        self.widgets['entry_name'] = gtk.Entry()
        self.widgets['table'].attach(self.widgets['entry_name'],
            1,3, 1,2, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_type'].set_mnemonic_widget(self.widgets['combobox_type'])
        self.widgets['entry_command'] = gtk.Entry()
        self.widgets['table'].attach(self.widgets['entry_command'],
            1,2, 2,3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_command'].set_mnemonic_widget(self.widgets['entry_command'])
        self.widgets['button_command'] = gtk.Button(label=_("_Browse"))
        self.widgets['button_command'].set_use_underline(True)
        self.widgets['table'].attach(self.widgets['button_command'],
            2,3, 2,3, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['entry_description'] = gtk.Entry()
        self.widgets['table'].attach(self.widgets['entry_description'],
            1,3, 3,4, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_description'].set_mnemonic_widget(self.widgets['entry_description'])
        self.widgets['entry_uninstall'] = gtk.Entry()
        self.widgets['table'].attach(self.widgets['entry_uninstall'],
            1,3, 4,5, xoptions=gtk.EXPAND|gtk.FILL, yoptions=gtk.EXPAND|gtk.FILL)
        self.widgets['label_uninstall'].set_mnemonic_widget(self.widgets['entry_uninstall'])
        
        """ Setup signal handlers """
        self.widgets['button_icon'].connect('clicked', self._on_button_icon_clicked)
        self.widgets['button_command'].connect('clicked', self._on_button_command_clicked)
        self.widgets['dialog'].show_all()
        
        self.widgets['entry_name'].grab_focus()
        self.widgets['dialog'].set_default_response(gtk.RESPONSE_OK)
        
        self.widgets['dialog'].connect('response', self._on_dialog_response)
        
        self.widgets['filechooserdialog_program_icon'] = common.filechooserdialog_new_with_filters(
            title = _("Choose an executable or an image"),
            parent = _PARENT_WINDOW,
            file_types = ['all', 'images', 'windows_executables', 'windows_executables_and_images'],
            default_filter = 'windows_executables_and_images',
            on_response_func = self._on_filechooser_icon_accept)
        
        self.widgets['filechooserdialog_program_command'] = common.filechooserdialog_new_with_filters(
            title = _("Choose an executable"),
            parent = _PARENT_WINDOW,
            file_types = ['all', 'windows_executables'],
            default_filter = 'windows_executables',
            on_response_func = self._on_filechooser_command_accept)
    
    def _update_interface_values(self):
        icon = common.pixbuf_new_from_program_data(self.program_data, size=self.widgets['image_icon'].get_pixel_size())
        command = description = uninstall = ''
        terminal = False
        for key in ['vineyardcommand', 'programcommand', 'programexe']:
            if key in self.program_data:
                command = self.program_data[key]
        for key in ['vineyarddescription', 'description']:
            if key in self.program_data:
                description = self.program_data[key]
        for key in ['vineyardterminal', 'programterminal']:
            if key in self.program_data:
                terminal = self.program_data[key]
        for key in ['vineyarduninstall', 'uninstall']:
            if key in self.program_data:
                uninstall = self.program_data[key]
        self.widgets['entry_name'].set_text(self.program_data['name'])
        self.icon_filename = None
        if icon:
        	self.widgets['image_icon'].set_from_pixbuf(icon)
        if terminal == True:
            self.widgets['combobox_type'].set_active(1)
        else:
            self.widgets['combobox_type'].set_active(0)
        self.widgets['entry_command'].set_text(command)
        self.widgets['entry_description'].set_text(description)
        self.widgets['entry_uninstall'].set_text(uninstall)
    
    
    def _on_dialog_response(self, dialog, response):
        if response == gtk.RESPONSE_OK:
            """ Get the new values from the dialog """
            name = self.widgets['entry_name'].get_text()
            icon = self.icon_filename
            command = self.widgets['entry_command'].get_text()
            description = self.widgets['entry_description'].get_text()
            terminal = self.widgets['combobox_type'].get_active() == 1
            uninstall = self.widgets['entry_uninstall'].get_text()
            
            """ Add the new values to the program dict """
            self.program_data.update({'name': name, 'programcommand': command, 'description': description, 'terminal': terminal, 'uninstall': uninstall})
            if icon != None:
                self.program_data.update({'icon': icon})
            #print "Added/Updated program info: %s" % self.program_data

            """ Save them in the registry """
            wine.programs.setProgramOptions(self.program_data['_registrykey'], name=name, icon=icon, command=command, description=description, terminal=terminal, uninstall=uninstall)

            """ Update our internal info """
            if self.program_id == None or self.program_id in self.settings['programs']:
                self.settings['programs'][self.program_id] = self.program_data
            elif program_id in self.settings['menuprograms']:
                self.settings['menuprograms'][self.program_id] = self.program_data

            """ Update the listview with the new info """
            treeview_model = self.parent_class.widgets['treeview'].get_model()
            programpath = common.model_get_path_from_string(
                model = treeview_model,
                string = self.program_id,
                column_num = 1,
                startswith = True
            )
            
            title = self._get_markup_from_program_data({'name': name, 'description': description})
            
            pixbuf = common.pixbuf_new_from_program_data(self.program_data, force_update=True)
            
            if programpath:
                treeview_model[programpath][1] = title
                if icon != None:
                    treeview_model[programpath][0] = pixbuf
            else:
                """ This is an new/added program """
                """ First get the list from the treeview """
                programnames = [ row[1].split('\n')[0] for row in treeview_model ]
                """ The isolate the first section, meaning the "Registered Programs" section """
                programnames = '\n'.join(programnames).split('<header>')[1].split('\n')[1:-1]
                """ Add the name of our new program and sort the list to get
                    the index of where we should append the new program """
                programnames.append(name)
                index = sorted(programnames).index(name)
                iter = treeview_model.get_iter((index,))
                treeview_model.insert_after(iter, [pixbuf, name])
                
            del self.parent_class.program_editors[self.program_id]
        dialog.hide()
    
    def _on_button_icon_clicked(self, button):
        path = self._get_directory_of_command(self.widgets['entry_command'].get_text())
        self.widgets['filechooserdialog_program_icon'].set_current_folder(path)
        self.widgets['filechooserdialog_program_icon'].show_all()
    
    def _on_filechooser_icon_accept(self, dialog, response):
        if response == gtk.RESPONSE_OK:
            self.icon_filename = filename = dialog.get_preview_filename()
            size = self.widgets['image_icon'].get_pixel_size()
            pixbuf = common.pixbuf_new_from_any_file(filename, size=size, only_programs=False)
            self.widgets['image_icon'].set_from_pixbuf(pixbuf)
        dialog.hide()
    
    def _on_button_command_clicked(self, button):
        path = self._get_directory_of_command(self.widgets['entry_command'].get_text())
        self.widgets['filechooserdialog_program_command'].set_current_folder(path)
        self.widgets['filechooserdialog_program_command'].show_all()
    
    def _on_filechooser_command_accept(self, dialog, response):
        if response == gtk.RESPONSE_OK:
            self.widgets['entry_command'].set_text(dialog.get_filename())
        dialog.hide()
    
    def _get_directory_of_command(self, command=''):
        path = None
        if len(command):
            """ If this is a UNIX path """
            if command[0] == '/':
                path = os.path.dirname(command)
            else:
                command = wine.util.splitString(command)[0]
                path = os.path.dirname(wine.util.wintounix(command))
            if not os.path.isdir(path):
                path = None
        if path == None:
            """ Set the directory to the first drive of the configuration """
            path = wine.drives.getMainDrive(drives=self.settings['drives'])['mapping']
        return path
        

class RunAndMonitorProgram:
    def __init__(self, command, name):
        process = wine.run(command)
        RUNNING_PROGRAMS[process.pid] = process
        RUNNING_PROGRAMS[process.pid].name = name
        gobject.timeout_add_seconds(1, self._start_monitoring)
        self.pid = process.pid

    def _start_monitoring(self):
        process = RUNNING_PROGRAMS[self.pid]
        process.poll()
        if process.returncode != None:
            if process.returncode != 0:
                dialog = gtk.MessageDialog(parent=_PARENT_WINDOW,
                                           flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                                           type=gtk.MESSAGE_ERROR,
                                           buttons=gtk.BUTTONS_OK)
                dialog.set_icon_name('vineyard-preferences')
                dialog.set_markup('<span weight="bold" size="larger">%s</span>' % \
                    _("The program exited with an error"))
                dialog.format_secondary_markup(unicode('The program <b>%(program)s</b> did ' + \
                    'not exit cleanly and any work or progress you may have ' + \
                    'done in the program may have been lost.') % \
                    {'program': common.escape_xml(process.name)})
                settings = gtk.settings_get_default()
                settings.set_property('gtk-alternative-button-order', True)
                button_info = gtk.Button("More info")
                dialog.add_action_widget(button_info, 10)
                dialog.set_alternative_button_order([10, gtk.RESPONSE_OK])
                dialog.set_default_response(gtk.RESPONSE_OK)
                dialog.connect("response", self._on_dialog_main_response)
                dialog.show_all()
            return False
        else:
            return True

    def _on_dialog_main_response(self, dialog, response):
        dialog.destroy()
        process = RUNNING_PROGRAMS[self.pid]
        errortext = process.stderr.readlines()
        if response == 10:
            show_explanation = False
            # See if we can find the cause of the error
            missing_dlls = []
            for line in errortext:
                match = re.match(r'(?i)^err:module:import_dll .*? (\w+\.\w{3})', line)
                if match:
                    missing_dlls.append(match.groups()[0].lower())
            missing_dlls = list(set(missing_dlls))
            # We found some missing dlls, alter display to match and find detailed information
            if len(missing_dlls):
                missing_packages = []
                libraryPackages = wine.libraries.libraryPackages
                text = _("A possible cause of the error could be one or more of the following:")+"\n"
                for dll in missing_dlls:
                    text += "\n\t"+_("The library file (DLL) <b>%s</b> wasn't found.") % dll
                    package = [ k for (k, v) in libraryPackages.iteritems() if dll in v ]
                    if len(package):
                        missing_packages.append(package[0])
                # Remove duplicate package findings
                missing_packages = list(set(missing_packages))
                if len(missing_packages):
                    if len(missing_packages) == 1:
                        text += str("\n\n"+_("Based on this output it looks like " + \
                            "you need to install <b>%(package)s</b> in order " + \
                            "to get <b>%(program)s</b> to run successfully.")) % \
                            {'package': missing_packages[0], 'program': common.escape_xml(process.name)}
                    else:
                        list_of_packages = _(", ").join([ "<b>%s</b>" % i for i in missing_packages[:-1] ])+ \
                                           _(" and ")+missing_packages[-1]
                        text += str("\n\n"+_("Based on this output it looks like " + \
                            "you need to install %(packages)s in order to get " + \
                            "<b>%(program)s</b> to run successfully.")) % \
                            {'packages': list_of_packages, 'program': common.escape_xml(process.name)}
                #self.widgets['label-programerrorexplanation'].set_line_wrap(True)
                #self.widgets['label-programerrorexplanation'].set_markup(text)
                #self.widgets['label-programerrorexplanation'].show()
                #self.widgets['vbox-programerrorexplanation'].show_all()
                #textview = self.widgets['textview-programerrorexplanation']
                show_explanation = True
            # We didn't find anything, just display the raw output
            else:
                #self.widgets['label-programerrortitle'].set_markup('<span weight="bold" size="larger">%s</span>' % _("When running \"%s\" Wine returned the following errors:") % process.name)
                #self.widgets['vbox-programerrornoexplanation'].show_all()
                #textview = self.widgets['textview-programerror']
                text = _("When running <b>%s</b> Wine returned the following errors:") % common.escape_xml(process.name)

            #self.widgets['dialog-programerror'].set_title(_("Error information for %s") % process.name)
            #self.widgets['dialog-programerror'].set_sensitive(False)
            #self.widgets['dialog-programerror'].show_all()
            #if show_explanation:
            #    self.widgets['vbox-programerrornoexplanation'].hide()
            #else:
            #    self.widgets['vbox-programerrorexplanation'].hide()
            #textview.get_buffer().set_text("\n".join([ i.strip() for i in errortext ]).replace('\r', ''))
            #textview.modify_font(pango.FontDescription('monospace'))
            #self.widgets['dialog-programerror'].set_sensitive(True)
            self._show_dialog_program_error(process.name,
                text,
                "\n".join([ i.strip() for i in errortext ]).replace('\r', ''),
                show_explanation)
        else:
            sys.stdout.write("\n".join([ i.strip() for i in errortext ]).replace('\r', ''))
    
    def _show_dialog_program_error(self, program_name, text, error, show_explanation=False):
        self._error_dialog = gtk.Dialog(_("Error information for %s") % program_name,
            parent = _PARENT_WINDOW,
            flags = gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR,
            buttons = (gtk.STOCK_OK, gtk.RESPONSE_OK))
        
        hbox = gtk.HBox(spacing=6)
        
        icon = gtk.Image()
        icon.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)
        icon.set_alignment(0.5, 0.0)
        
        hbox.pack_start(icon, expand=False, fill=True)
        
        vbox = gtk.VBox()
        
        label = gtk.Label()
        label.set_line_wrap(True)
        label.set_alignment(0.0, 0.5)
        #label.set_markup('<span weight="bold" size="larger">%s</span>' % text)
        label.set_markup('%s' % text)
        
        vbox.pack_start(label, expand=False, fill=True)
        
        scrolledwindow = gtk.ScrolledWindow()
        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolledwindow.set_size_request(
            common.widget_get_char_width(scrolledwindow)*80,
            common.widget_get_char_height(scrolledwindow)*6)
        
        textview = gtk.TextView()
        textview.get_buffer().set_text(error)
        textview.modify_font(pango.FontDescription('monospace'))
        scrolledwindow.add(textview)
        
        if not show_explanation:
            vbox.pack_start(scrolledwindow, expand=True, fill=True)
        else:
            expander = gtk.Expander(_("Detailed error information"))
            expander.add(scrolledwindow)
            vbox.pack_start(expander, expand=True, fill=True)
        
        hbox.pack_start(vbox, expand=True, fill=True)
        
        self._error_dialog.vbox.pack_start(hbox, expand=True, fill=True)
        
        self._error_dialog.connect('destroy', self._on_dialog_program_error_response)
        self._error_dialog.connect('response', self._on_dialog_program_error_response)
        
        self._error_dialog.show_all()

    def _on_dialog_program_error_response(self, *args):
        self._error_dialog.destroy()
