#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry

registry = _registry.Registry()

class Audio:
	def __init__(self):
		self._AudioAccels = {
			"default": "Full",
			"full": "Full",
			"standard": "Standard",
			"basic": "Basic",
			"emulation": "Emulation"
		}
		self.Accels = self._AudioAccels.keys()
		self.Rates = [48000, 44100, 22050, 16000, 11025, 8000]
		self.Bits = [8, 16]
		self._AudioDrivers = {
			"alsa":				"alsa",
			"oss":				"oss",
			"coreaudio":	"coraudio",
			"jack":				"jack",
			"nas":				"nas",
			"esd":				"esd",
			"audioio":		"audioio"
		}
		self.Drivers = self._AudioDrivers.keys()
	
	def getAcceleration(self, program=None):
		if program:
			level = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program, "HardwareAcceleration", quiet=True)
		else:
			level = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound', "HardwareAcceleration", quiet=True)
		if level:
			return level
		else:
			return self._AudioAccels['default']
	
	def setAcceleration(self, accel, program=None):
		"""
			Sets the level of audio acceleration to use
			accel can be one of ["full", "standard", "basic", "emulation"]
		"""
		if program:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {"HardwareAcceleration": self._AudioAccel[accel]}})
		else:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {"HardwareAcceleration": self._AudioAccels[accel]}})
	
	def getSampleRate(self, program=None):
		if program:
			rate = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program, "DefaultSampleRate", quiet=True)
		else:
			rate = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound', "DefaultSampleRate", quiet=True)
		if rate:
			return rate
		else:
			return "44100"
	
	def setSampleRate(self, rate, program=None):
		"""
			Sets the DirectSound audio sample rate
		"""
		if program:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {"DefaultSampleRate": rate}})
		else:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {"DefaultSampleRate": rate}})
	
	def getBitDepth(self, program=None):
		if program:
			bits = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program, "DefaultBitsPerSample", quiet=True)
		else:
			bits = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound', "DefaultBitsPerSample", quiet=True)
		if bits:
			return bits
		else:
			return "16"
	
	def setBitDepth(self, bits, program=None):
		"""
			Sets the DirectSound audio bit depth
		"""
		if program:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {"DefaultBitsPerSample": bits}})
		else:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {"DefaultBitsPerSample": bits}})
	
	def getEmulateDriver(self, program=None):
		if program:
			state = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program, "EmulDriver", quiet=True)
		else:
			state = registry.get('HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound', "EmulDriver", quiet=True)
		if state and "y" in state.lower():
			return True
		else:
			return False
	
	def setEmulateDriver(self, state, program=None):
		"""
			Sets wether DirectSound should emulate a hardware driver
		"""
		if state:
			state = "Y"
		else:
			state = "N"
		
		if program:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\AppDefaults\\%s\\DirectSound' % program: {"EmulDriver": state}})
		else:
			registry.set({'HKEY_USERS\\S-1-5-4\\Software\\Wine\\DirectSound': {"EmulDriver": state}})
	
	def getEnabledDrivers(self):
		drivers = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Drivers', "Audio", quiet=True)
		if drivers not in [ None, {} ]:
			return drivers.split(',')
		else:
			return []
	
	def setEnabledDrivers(self, drivers):
		"""
			Sets the list of enabled audio drivers
			disabling all others
		"""
		registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})
	
	def enableDriver(self, driver):
		"""
			Adds an audio driver to the list of enabled audio drivers
			driver can be one of ["alsa", "oss", "coreaudio", "jack", "nas", "esd", "audioio"]
		"""
		drivers = filter(len, self.getEnabledDrivers())
		if self._AudioDrivers[driver] not in drivers:
			drivers.append(self._AudioDrivers[driver])
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})
	
	def disableDriver(self, driver):
		"""
			Removes an audio driver from the list of enabled audio drivers
			driver can be one of ["alsa", "oss", "coreaudio", "jack", "nas", "esd", "audioio"]
		"""
		drivers = self.getEnabledDrivers()
		if self._AudioDrivers[driver] in drivers:
			drivers.remove(self._AudioDrivers[driver])
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Drivers': {"Audio": ','.join(drivers)}})
