#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry, util
import os, subprocess

registry = _registry.Registry()

class Drives:
	def driveMappingToReg(self, driveletter, mapping):
		hexmapping = util.utf8tohex(mapping).upper()
		hexletter = "00000000-0000-0000-0000-0000000000%s" % driveletter.upper().encode('hex')
		
		registry.set({'HKEY_LOCAL_MACHINE\\System\\MountedDevices': {
			"\\\\??\\\\Volume{%s}" % hexletter: "hex:%s" % hexmapping,
			"\\\\DosDevices\\\\%s:" % driveletter.upper(): "hex:%s" % hexmapping
		}})
		registry.set({'HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives': {
			"%s:" % driveletter.upper(): "hd"
		}})
	
	def getMainDrive(self, drives=None):
	    if drives == None:
	        drives = self.get()
	    return sorted([ (k,v) for (k,v) in drives.iteritems() ])[0][1]
	
	def add(self, driveletter, mapping, *args):
		driveletter = driveletter.lower()
		drivemapping = os.path.normpath("%s/dosdevices/%s:" % (os.environ['WINEPREFIX'], driveletter))
		mapping = os.path.normpath(mapping)
		
		# If the directory already exists and is a link that is already set to what we need
		if not (os.path.exists(drivemapping) and os.path.islink(drivemapping) and os.path.realpath(drivemapping) == os.path.realpath(mapping)):
			# Else set up the mapping
			if os.path.exists(drivemapping) and os.path.islink(drivemapping):
				os.remove(drivemapping)
		
			os.symlink(mapping, drivemapping)
		
		self.driveMappingToReg(driveletter, mapping)
	
		returncode = subprocess.call(["wine", "net", "stop", "mountmgr"], env=os.environ)
		returncode = subprocess.call(["wine", "net", "start", "mountmgr"], env=os.environ)
		
		# If label or serial is given
		if len(args):
			# Set label and serial (if given)
			if len(args[0]):
				f = file("%s/.windows-label" % drivemapping, 'w')
				f.write("%s\n" % args[0])
				f.close()
			elif os.path.exists("%s/.windows-label" % drivemapping):
				os.remove("%s/.windows-label" % drivemapping)
			if len(args) > 1:
				if len(args[1]):
					f = file("%s/.windows-serial" % drivemapping, 'w')
					f.write("%s\n" % args[1])
					f.close()
				elif os.path.exists("%s/.windows-serial" % drivemapping):
					os.remove("%s/.windows-serial" % drivemapping)
	
	def remove(self, driveletter):
		if os.path.exists( os.path.expandvars("%s/dosdevices/%s:" % (os.environ['WINEPREFIX'], driveletter.lower())) ):
			os.remove(os.path.expandvars("%s/dosdevices/%s:" % (os.environ['WINEPREFIX'], driveletter.lower())))
		elif os.path.exists( os.path.expandvars("%s/dosdevices/%s:" % (os.environ['WINEPREFIX'], driveletter.upper())) ):
			os.remove(os.path.expandvars("%s/dosdevices/%s:" % (os.environ['WINEPREFIX'], driveletter.upper())))
			
		registry.set({'HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives': {"%s:" % driveletter: None}})
		
		hexletter = "00000000-0000-0000-0000-0000000000%s" % driveletter.upper().encode('hex')
		registry.set({'HKEY_LOCAL_MACHINE\\System\\MountedDevices': {
			"\\\\??\\\\Volume{%s}" % hexletter: None,
			"\\\\DosDevices\\\\%s:" % driveletter.upper(): None
		}})
		
		
	def setType(self, driveletter, type):
		# Make sure we use the same caps for the driveletter as the reg
		typesraw = registry.get('HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives', quiet=True)
		for key,value in typesraw.iteritems():
			if key.lower == driveletter.lower():
				driverletter = key
		# Save it
		if type in ["hd", "network", "cdrom", "floppy"]:
			registry.set({'HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives': {"%s:" % driveletter: type}})
		else:
			registry.set({'HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives': {"%s:" % driveletter: None}})
	
	def list(self):
		"""
			Prints a list of drive mappings similar to:
			
			Drive:		C
			Mapping:	/home/user/.wine/drive_c
			Label:		Windows
			Serial:		0
			
			Label and Serial are only printed if they are set
		"""
		
		mappings = ""
		
		drives = registry.get('HKEY_LOCAL_MACHINE\\System\\MountedDevices')
		typesraw = registry.get('HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives')
		types = {}
		for key,value in typesraw.iteritems():
			types[key.lower()] = value.lower()
		
		for key in drives.keys():
			if key.lower().replace('\\\\','\\').startswith('\\dosdevices'):
				drive = key.replace('\\\\','\\').lower().split('\\dosdevices\\')[1]
				drivepath = drives[key]
				
				if drivepath.startswith("hex:"):
					drivepath = util.hextoutf8(drivepath)
				if drivepath.startswith("./"):
					drivepath = "%s/dosdevices/%s" % (os.environ['WINEPREFIX'], drivepath[2:])
				elif drivepath.startswith("../"):
					drivepath = "%s/dosdevices/../%s" % (os.environ['WINEPREFIX'], drivepath[2:])
				
				drivepath = drivepath.replace('\0','')
				
				# if .wine/dosdevices/DRIVE: doesn't exist, Wine won't use it, continue
				if not os.path.exists("%s/dosdevices/%s" % (os.environ['WINEPREFIX'], drive.lower())) and not os.path.exists("%s/dosdevices/%s" % (os.environ['WINEPREFIX'], drive.upper())):
					continue
				
				mappings += "Drive:\t\t%s\n" % drive.upper()
				mappings += "Mapping:\t%s\n" % os.path.realpath(drivepath)
				
				if drive.lower() in types.keys():
					mappings += "Type:\t\t%s\n" % types[drive.lower()]
				
				if os.path.exists("%s/.windows-label" % drivepath):
					f = file("%s/.windows-label" % drivepath)
					serial = f.read().replace('\n','').replace('\r','')
					f.close()
					mappings += "Label:\t\t%s\n" % serial
				if os.path.exists("%s/.windows-serial" % drivepath):
					f = file("%s/.windows-serial" % drivepath)
					serial = f.read().replace('\n','').replace('\r','')
					f.close()
					mappings += "Serial:\t\t%s\n" % serial
				mappings += "\n"
		return mappings[:-1]
	
	def get(self):
		drives = {}
		linenumber = 0
		drive = None
		driveslist = self.list()
		for line in driveslist.split("\n"):
			linenumber += 1
			if line.startswith("Drive:"):
				if drive != None:
					drives[drive] = info
				drive = line.split("\t")[-1]
				info = {'label': '', 'serial': '', 'type': ''}
			elif line.startswith("Mapping:"):
				info['mapping'] = line.split("\t")[-1]
			elif line.startswith("Type:"):
				info['type'] = line.split("\t")[-1]
			elif line.startswith("Label:"):
				info['label'] = line.split("\t")[-1]
			elif line.startswith("Serial:"):
				info['serial'] = line.split("\t")[-1]
			else:
				if drive != None:
					drives[drive] = info
				linenumber -=1
		return drives
	
	def getShowDotFiles(self):
		settings = registry.get('HKEY_CURRENT_USER\\Software\\Wine', quiet=True)
		return "ShowDotFiles" in settings and "Y" in settings["ShowDotFiles"]
	
	def setShowDotFiles(self, value):
		if value:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine': {"ShowDotFiles": "Y"} })
		else:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine': {"ShowDotFiles": "N"} })
	
	def getAutoDetect(self):
		# Remove existing drives (wine may complain)
		
		#for filename in os.listdir("%s/dosdevices/" % os.environ['WINEPREFIX']):
		#	if filename.lower() != 'c:':
		#		os.remove("%s/dosdevices/%s" % (os.environ['WINEPREFIX'], filename))
		#drives = registry.get('HKEY_LOCAL_MACHINE\\System\\MountedDevices')
		#typesraw = registry.get('HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives')
		#for drive in drives.keys():
		#	if drive.lower() != '\\??\\volume{00000000-0000-0000-0000-000000000043}' and drive.lower() != '\dosdevices\c:':
		#		drives[drive] = None
		#for drive in typesraw.keys():
		#	if drive.lower != 'c:':
		#		typesraw[drive] = None
		#registry.set({'HKEY_LOCAL_MACHINE\\System\\MountedDevices': drives})
		#registry.set({'HKEY_LOCAL_MACHINE\\Software\\Wine\\Drives': typesraw})
		
		# Get drives
		drives = {'C:': {'mapping': '%s/drive_c' % os.environ['WINEPREFIX'], 'type': 'hd'}, 'Z:': {'mapping': "/", 'type': 'hd'}}
		if os.path.exists("/proc/sys/dev/cdrom/info"):
			location = subprocess.Popen(["grep", "drive name:", "/proc/sys/dev/cdrom/info"], stdout=subprocess.PIPE, env=os.environ).communicate()[0]
			location = location.split("\t")[-1].strip()
			drives['D:'] = {
				'mapping': "/dev/%s" % location,
				'type': 'cdrom'
			}
		return drives
		
		# Add drives
		#for driveletter, mapping in drives.iteritems():
		#	self.add(driveletter, mapping)
		#	self.setType(driveletter, 'hd')
		#
		#if 'D' in drives:
		#	self.setType('D', 'cdrom')
	
