#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import os
import util, parsers

import subprocess

import logging

class Registry:
    def __init__(self):
        self.registryCache = None
        self.logging = logging.getLogger("python-wine.programs")
        self.debug = self.logging.debug
        self.error = self.logging.error
    
    def __branch_to_dict(self, branch, dict):
        parents = []
        for path in filter(len, branch.split('\\')):
            pathname = 'dict%s' % ''.join([ '[\'%s\']' % i for i in parents ])
            if path not in eval(pathname) or type(eval(pathname)) != type({}):
                exec('%s[\'%s\'] = {}' % (pathname, path))
            parents.append(path)
        return dict
    
    def __set_branch_values(self, branch, dict, items):
        # Make sure the dict keys exist
        dict = self.__branch_to_dict(branch, dict)
        # Set the value
        pathname = 'dict%s' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\')) ])
        for key, value in items.iteritems():
            try:
                exec ('%s[\'%s\'] = value' % (pathname, key.replace('\\', '\\\\').replace("'", '\\')))
            except ValueError:
                self.error("Error in registry! Pathname: %s\tKey: %s\tValue: %s" % (pathname, key, value))
        return dict
    
    def _writeRegistry(self, registrycontent):
        regfilename = util.tempname('vineyard-setregistry-', '.reg')
        regfile = open(regfilename, "w")
        regfile.write(registrycontent)
        regfile.close()
        returnvalue = subprocess.call(["regedit", regfilename], env=os.environ)
        #os.remove(regfilename)
        return returnvalue
    
    def __get_branch(self, branch=None, quiet=True):
        if branch.startswith('HKEY_LOCAL_MACHINE') and branch != 'HKEY_LOCAL_MACHINE\\System\\MountedDevices':
            registry_file = open('%s/system.reg' % os.environ['WINEPREFIX'], 'r')
            registry = registry_file.read()
            registry_file.close()
            #print "Using system.reg file as registry."
            return ('HKEY_LOCAL_MACHINE', registry)
        elif branch.startswith('HKEY_CURRENT_USER'):
            registry_file = open('%s/user.reg' % os.environ['WINEPREFIX'], 'r')
            registry = registry_file.read()
            registry_file.close()
            #print "Using user.reg file as registry."
            return ('HKEY_CURRENT_USER', registry)
        elif branch.startswith('HKEY_USERS\\.Default'):
            registry_file = open('%s/userdef.reg' % os.environ['WINEPREFIX'], 'r')
            registry = registry_file.read()
            registry_file.close()
            #print "Using userdef.reg file as registry."
            return ('HKEY_USERS\\.Default', registry)

        #print "Using regedit to read registry."
        self._initWine()
        if branch == None:
            process = subprocess.Popen(["regedit", "/E", "-"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
            branch = ''
        else:
            process = subprocess.Popen(["regedit", "/E", "-", branch], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
        registry = ''
        buffer = process.stdout.readline()
        while buffer != '':
            registry += buffer
            buffer = process.stdout.readline()
        #if process.returncode and not quiet:
        if registry == '' or registry.strip() == 'Success':
            if not quiet:
                self.error("Warning: Registry branch '%s' doesn't exist." % branch)
            return ''
        return registry
    
    def __branchpath_in_dict(self, dict, branch):
        try:
            self.__get_dict_from_branchpath(dict, branch)
            return True
        except:
            return False
    
    def __get_dict_from_branchpath(self, dict, branch):
        try:
            return eval('dict%s' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\'))]))
        except NameError:
            return False
    
    def __set_dict_from_branchpath(self, dict, branch, value):
        exec('dict%s = value' % ''.join([ '[\'%s\']' % i for i in filter(len, branch.split('\\'))]))
        return dict
    
    def get(self, branch=None, findkey=None, quiet=True):
        if findkey:
            fullbranch = '%s\\%s' % (branch, findkey)
        else:
            fullbranch = branch
        
        branch_output = self.__get_branch(branch)
        if type(branch_output) == tuple:
            reg = parsers.parseRegistry(branch_output[1], branch_output[0])
        else:
            reg = parsers.parseRegistry(branch_output)
        
        if reg == None:
            reg = {}
        else:
            try:
                reg = self.__get_dict_from_branchpath(reg, branch)
            except KeyError:
                if findkey:
                    return None
                else:
                    return {}
            
        if findkey:
            if findkey in reg:
                retvalue = reg[findkey]
            else:
                retvalue = None
        else:
            retvalue = reg
        
        return retvalue

    def set(self, branches):
        # Set registry values
        # Branches should be a dict similar to:
        # {
        #  'HKEY_CURRENT_USER\\Software\\Wine':
        #   {"Version": "winnt"},
        # 'HKEY_USERS\\S-1-5-4\\Control Panel\\Desktop':
        #   {"FontSmoothing": "2"}
        # }
        self._initWine()
        
        registry = "REGEDIT4\n"

        definedbranches = []
        for b in sorted(branches.keys()):
            branchwalk = b.split('\\')
            for i in range(len(branchwalk)-1):
                branch = "\n[%s\\%s]\n" % (branchwalk[0], '\\'.join(branchwalk[1:i+2]))
                if branch not in definedbranches:
                    registry += branch
                    definedbranches.append(branch)
            # Add the values
            for key,value in sorted(branches[b].iteritems()):
                if value == '-' or value == None:
                    registry += "\"%s\"=-\n" % str(key).encode('utf-8').encode('string_escape')
                elif str(value).startswith('hex:') or str(value).startswith('dword:'):
                    registry += "\"%s\"=%s\n" % (str(key).encode('utf-8').encode('string_escape'), str(value))
                else:
                    try:
                        registry += "\"%s\"=\"%s\"\n" % (str(key).encode('utf-8').encode('string_escape'), str(value).encode('utf-8').encode('string_escape').replace('"', '\\"'))
                    except TypeError:
                        print "Error:",key,value
        registry += "\n"
        self._writeRegistry(registry)
    
    def _initWine(self):
        self._wineprocess = subprocess.Popen("wineserver -p10", shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ)
