#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry, util
import os, subprocess

registry = _registry.Registry()

class ShellFolders:
	def get(self):
		folders = {}
		userpath = registry.get('HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Session Manager\\Environment', 'APPDATA')
		for foldername in ["Desktop", "My Documents", "My Pictures", "My Music", "My Videos"]:
			if userpath:
				folders[foldername] = os.path.realpath("%s/drive_c/%s/%s" % (os.environ['WINEPREFIX'], "/".join(userpath[3:].split("\\")[:-1]), foldername))
			else:
				folders[foldername] = util.getRealHome()
		return folders
	
	def set(self, foldername, target):
		if foldername.lower() in ["desktop", "my documents", "my pictures", "my music", "my videos"]:
			# Camelcase the folder
			foldername = " ".join([ i.capitalize() for i in foldername.split(" ") ])
			userpath = registry.get('HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Session Manager\\Environment', 'APPDATA')
			folderpath = "%s/drive_c/%s/%s" % (os.environ['WINEPREFIX'], "/".join(userpath[3:].split("\\")[:-1]), foldername)
			try:
				os.remove(folderpath)
			except OSError:
				try:
					os.rmdir(folderpath)
				except OSError:
					print "Skipping %s, there are files in the directory" % os.path.basename(folderpath)
			os.symlink(target, folderpath)
	
	def automap(self):
		HOME = util.getRealHome()
		# Iterate the XDG User Dirs and set Wine's ShellFolders up to match, defaulting to $HOME if there is no matching XDG Dir
		for dirname in [('DESKTOP', 'Desktop'), ('DOCUMENTS', 'My Documents'), ('MUSIC', 'My Music'), ('PICTURES', 'My Pictures'), ('VIDEOS', 'My Videos')]:
			xdgdir = self._get_xdg_dir(dirname[0])
			if xdgdir:
				self.set(dirname[0], xdgdir)
			else:
				# First try, fx. "$HOME/My Documents", then "$HOME/Documents" else just link to "$HOME"
				if os.path.exists("%s/%s" % ( HOME, dirname[0].capitalize() )):
					self.set(dirname[0], "%s/%s" % ( HOME, dirname[0].capitalize() ))
				elif os.path.exists("%s/%s" % ( HOME, dirname[1] )):
					self.set(dirname[0], "%s/%s" % ( HOME, dirname[1] ))
				else:
					self.set(dirname[0], HOME)
	
	def _get_xdg_dir(self, name):
		HOME = util.getRealHome()
		lines = []
		for filename in ["$XDG_CONFIG_HOME/user-dirs.dirs", "%s/.config/user-dirs.dirs" %HOME]:
			if os.path.exists(os.path.expandvars(filename)):
				f = open(filename)
				lines = f.readlines()
				f.close()
				break
		for line in lines:
			if line.startswith('XDG_'+name.upper()):
				return os.path.normpath(os.path.expandvars(line.split('=')[1].strip()[1:-1]))
		return None
			
