#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry

class Version:
	def __init__(self):
		self.registry = _registry.Registry()
		
		self.versions = ['win20', 'win30', 'win31', 'win95', 'win98', 'winme', 'nt351', 'nt40', 'win2k', 'winxp', 'win2003', 'vista', 'win2008']
		self.windowsversions = {
			"win2008":		["Windows 2008",		6,  0, 0x1771,"WIN32_NT",				"Service Pack 1",		0, 0, "ServerNT"],
			"vista":		["Windows Vista",		6,  0, 0x1770,"WIN32_NT",				" ",					0, 0, "WinNT"],
			"win2003":		["Windows 2003",		5,  2, 0xECE, "WIN32_NT",				"Service Pack 1",		1, 0, "ServerNT"],
			"winxp":		["Windows XP",			5,  1, 0xA28, "WIN32_NT",				"Service Pack 2",		2, 0, "WinNT"],
			"win2k":		["Windows 2000",		5,  0, 0x893, "WIN32_NT",				"Service Pack 4",		4, 0, "WinNT"],
			"winme":		["Windows ME",			4, 90, 0xBB8, "WIN32_WINDOWS",			" ",					0, 0, ""],
			"win98":		["Windows 98",			4, 10, 0x8AE, "WIN32_WINDOWS",			" A ",					0, 0, ""],
			"win95":		["Windows 95",			4,  0, 0x3B6, "WIN32_WINDOWS",			"",						0, 0, ""],
			"nt40":			["Windows NT 4.0",		4,  0, 0x565, "WIN32_NT",				"Service Pack 6a",		6, 0, "WinNT"],
			"nt351":		["Windows NT 3.5",		3, 51, 0x421, "WIN32_NT",				"Service Pack 2",		0, 0, "WinNT"],
			"win31":		["Windows 3.1",			2, 10,     0, "WIN32s",					"Win32s 1.3",			0, 0, ""],
			"win30":		["Windows 3.0",			3,  0,     0, "WIN32s",					"Win32s 1.3",			0, 0, ""],
			"win20":		["Windows 2.0",			2,  0,     0, "WIN32s",					"Win32s 1.3",			0, 0, ""]
		}
		self.wDescription = 0
		self.wMajorVersion = 1
		self.wMinorVersion = 2
		self.wBuildNumber = 3
		self.wPlatformId = 4
		self.wCSDVersion = 5
		self.wServicePackMajor = 6
		self.wServicePackMinor = 7
		self.wProductType = 8
		self.Key9x = "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion"
		self.KeyNT = "HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion"
		self.KeyProdNT = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ProductOptions"
		self.KeyWindNT = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Windows"
		self.KeyEnvNT  = "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Session Manager\\Environment"
	
	def get(self, program=None):
		if program:
			version = None
			programs = self.registry.get('HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults', quiet=True)
			for key in programs.keys():
				programs[key.lower()] = programs[key]
			if program.lower() in programs.keys() and 'Version' in programs[program.lower()]:
				version = programs[program.lower()]['Version']
			if version == None or version not in self.windowsversions.keys():
				return "winxp"
			else:
				return version
		else:
			version = self.registry.get('HKEY_CURRENT_USER\\Software\\Wine', "Version", quiet=True)
			if version:
				if version == None or version not in self.windowsversions.keys():
					return "winxp"
				else:
					return version
			else:
				return self._parseWineVersionFromRegistry()
	
	def set(self, version, program=False):
		if version.lower() == "default":
			version = "winxp"
		elif version not in self.versions:
			print "Error: Version should be one of the following: default, %s" % ", ".join(self.versions)
			sys.exit(1)
		
		if program:
			self.registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults\\%s' % program: {"Version": version} })
		else:
			self.setDefault(version)
	
	def setDefault(self, version):
		if version == "win2000":
			version = "win2k"
		shortversion = version
		version = self.windowsversions[version]
		      
		versionnumber9x = "%s.%s.%s" % (version[self.wMajorVersion], version[self.wMinorVersion], version[self.wBuildNumber])
		versionnumberNT = "%s.%s" % (version[self.wMajorVersion], version[self.wMinorVersion])
		
		if version[self.wPlatformId] == "WIN32_WINDOWS":
			registry = {
				self.Key9x: {
					"VersionNumber": versionnumber9x,
					"SubVersionNumber": version[self.wCSDVersion]
				},
				self.KeyNT: { "CSDVersion": None, "CurrentVersion": None, "CurrentBuildNumber": None },
				self.KeyProdNT: { "ProductType": None },
				self.KeyWindNT: { "CSDVersion": None },
				self.KeyEnvNT: { "OS": None },
				'HKEY_CURRENT_USER\\Software\\Wine': {"Version": None}
			}
		elif version[self.wPlatformId] == "WIN32_NT":
			registry = {
				self.KeyNT: {
					"CurrentVersion": versionnumberNT,
					"CSDVersion": version[self.wCSDVersion],
					"CurrentBuildNumber": version[self.wBuildNumber]
				},
				self.KeyProdNT: { "ProductType": version[self.wProductType] },
				self.KeyWindNT: { "CSDVersion": "dword:00000%s00" % version[self.wServicePackMajor] },
				self.KeyEnvNT: { "OS": "Windows_NT" },
				self.Key9x: { "VersionNumber": None, "SubVersionNumber": None },
				'HKEY_CURRENT_USER\\Software\\Wine': {"Version": None}
			}
		elif version[self.wPlatformId] == "WIN32s":
			registry = {
				self.KeyNT: { "CSDVersion": None, "CurrentVersion": None, "CurrentBuildNumber": None },
				self.KeyProdNT: { "ProductType": None },
				self.KeyWindNT: { "CSDVersion": None },
				self.KeyEnvNT: { "OS": None },
				self.Key9x: { "VersionNumber": None, "SubVersionNumber": None },
				'HKEY_CURRENT_USER\\Software\\Wine': {"Version": shortversion}
			}
		self.registry.set(registry)
	
	def _parseWineVersionFromRegistry(self):
		versionnt = self.registry.get(self.KeyNT, "CurrentVersion")
		version9x = self.registry.get(self.Key9x, "VersionNumber")
		best = "winxp" # hardcoded fallback (same as in winecfg)
		
		if versionnt:
			version = versionnt
			platform = "WIN32_NT"
			build = int(self.registry.get(self.KeyNT, "CurrentBuildNumber"))
		elif version9x:
			version = version9x
			platform = "WIN32_WINDOWS"
		else:
			return best
		
		if '.' in version:
			parts = version.split('.')
			if len(parts) == 3:
				build = int(parts[2])
			minor = int(parts[1])
		major = int(float(version))
		
		for winver,data in self.windowsversions.iteritems():
			if data[self.wPlatformId] != platform or data[self.wMajorVersion] != major:
				continue
			best = winver
			if data[self.wMinorVersion] == minor and data[self.wBuildNumber] == build:
				return winver
		return best

	def getPrograms(self):
		programs = self.registry.get('HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults', quiet=True)
		for key in programs.keys():
			programs[key.lower()] = programs[key]
		return programs
