#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import _registry

registry = _registry.Registry()

_openargs = []

def get(program=None):
	if program:
		program = program.lower()
		# Does this program use a virtual desktop?
		desktop = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults\\%s\\Explorer' % program.lower(), "Desktop", quiet=True)
		if desktop:
			desktop = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops', desktop, quiet=True)
			if desktop:
				return tuple([ int(i) for i in desktop.lower().split('x') ])
			else:
				return None
	else:
		# Does the default configuration use a virtual desktop?
		desktop = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Explorer', "Desktop", quiet=True)
		if desktop:
			desktop = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops', desktop, quiet=True)
			if desktop:
				return tuple([ int(i) for i in desktop.lower().split('x') ])
			else:
				return None

def set(boolean, size=None, program=None):
	"""
		Set whether Wine should emulate a virtual desktop
		
		@param boolean whether the virtual desktop should be used
		@type  boolean 
		@param size    the size of the the virtual desktop in pixels
		@type  tuple   (width,height) - defaults to 800x600 (like winecfg)
		@param program if set, the virtual desktop should only be set for this program
		@type  string  "explorer.exe"

	"""
	if program:
		program = program.lower()
	
	if size == None:
		if program:
			size = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops', program, quiet=True)
		else:
			size = registry.get('HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops', "Default", quiet=True)
		if not size:
			size = (800, 600)
		else:
			size = tuple([ int(i) for i in size.split('x') ])
	else:
		size = tuple(size)
	
	if boolean:
		if program:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops': {program: '%sx%s' % size}})
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults\\%s\\Explorer' % program : {"Desktop": program}})
		else:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Explorer\\Desktops': {"Default": '%sx%s' % size}})
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Explorer': {"Desktop": "Default"}})
	else:
		if program:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\AppDefaults\\%s\\Explorer' % program : {"Desktop": None}})
		else:
			registry.set({'HKEY_CURRENT_USER\\Software\\Wine\\Explorer': {"Desktop": None}})

def setOpen(name, size):
	_openargs.append("explorer")
	_openargs.append("/desktop=%s,%s" % ('-'.join(name.split(' ')), size.lower()))
