#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import os, re
from collections import defaultdict
import util

def parseRegistry(registry, base_path=None):
    registry = parseInput(registry)
    registry_dict = multidict()
    sections = re.findall(r'(?ms)^\[(?P<path>[^\]]+)\](?:\s\d+)?\s+(?P<section>.*?)(?=^\[)', registry+'\n[')
    for path, section in sections:
        values = re.findall(r'(?ms)^(?P<key>\"[^\"]*?\"|@)=(?P<value>.*?)(?=^\")', section+'"')
        section_dict = multidict()
        for key, value in values:
            """ Strip the start and end quotes """
            if key != '@':
                key = key[1:-1]
            value = parseValue(value)
            if type(value) == tuple:
                section_dict['_%s' % key] = value[0]
                section_dict[key] = value[1]
            else:
                section_dict[key] = value
        exec('registry_dict%s = section_dict' % ''.join([ '[\'%s\']' % i.encode('string_escape') for i in filter(len,path.split('\\'))]))
        #registry_dict = __set_branch_values(path, registry_dict, section_dict)
        #registry_dict[path] = section_dict
    if base_path:
        return {base_path: registry_dict.get_dict()}
    else:
        return registry_dict.get_dict()

def parseValue(value):
    """ See what type of value this is.
        For more info, see http://technet.microsoft.com/en-us/library/bb727154.aspx
    """
    original_value = None
    value_is_string = True
    value_type = value.split(':')[0].lower()
    if value_type.startswith('hex(2)'):
        """ Expanded String """
        try:
            new_value = util.hextoutf8(value)
            original_value = value
            value = new_value
        except UnicodeDecodeError:
            value_is_string = False
            print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
            print path, key, value_type, value
            exit(1)
    elif value_type.startswith('hex(7)'):
        """ Multi String """
        value_is_string = False
        try:
            value = filter(len, util.hextoutf8(value).split('\x00'))
        except TypeError:
            print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
            print path, key, value_type, value
            exit(1)
    elif value_type.startswith('hex'):
        """ HEX Value """
        value_is_string = False
        value = '%s:%s' % (value.split(':')[0], re.sub(r'(?i)[^a-z0-9,]','', ''.join(value.split(':')[1:])))
    elif value_type.startswith('dword'):
        """ Double Word (displayed as HEX usually) """
        value_is_string = False
        value = value.strip()
    elif value_type.startswith('null'):
        value_is_string = False
        value = None
    elif value_type.startswith('str(2)'):
        """ Expandable String """
        value = ':'.join(value.strip().split(':')[1:])
    elif value_type.startswith('quote') or value.startswith('str'):
        value = ':'.join(value.strip().split(':')[1:])
    if value_is_string:
        if value.strip().startswith('"') and value.replace('\x00','').strip().endswith('"'):
            value = '"'.join(value.strip().split('"')[1:-1])
        try:
            if type(value) == list:
                value = [ util.stringtoutf8(i) for i in value ]
            else:
                value = util.stringtoutf8(value)
        except UnicodeDecodeError:
            print "Couldn't read registry! This is a serious error, please report this error along with this text at https://bugs.launchpad.net/vineyard"
            print path, key, value_type, value.encode('hex')
            exit(1)
    if original_value != None:
        return (original_value, value)
    else:
        return value

def parseInput(input_object):
    """
        Takes either a file descriptor object, a file name or the contents of a
        file as an argument and returns the contents of it.
    """
    if type(input_object) == file:
        output = input_object.read()
    elif '\n' not in input_object and os.path.isfile(input_object):
        file_object = open(input_object, 'r')
        output = file_object.read()
        file_object.close()
    else:
        return input_object
    return output

class multidict(defaultdict):
    def __init__(self):
        self.default_factory = type(self)
    def get_dict(self):
        return self.__convert_to_dict(self)
    def __convert_to_dict(self, mdict):
        return_dict = {}
        for key,value in mdict.iteritems():
            if type(value) == type(self):
                return_dict[key] = self.__convert_to_dict(value)
            else:
                return_dict[key] = value
        return return_dict
