#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import os, random, string, re, fnmatch, subprocess

def hextoutf8(string):
	""" Remove any leading 'hex:' or 'hex(2):' or similar """
	if 'hex' in string[:8] and ':' in string[:8]:
		string = string.split(':')[1]
	""" Remove any formatting of the string """
	# Using string.replace is actually 3 times faster than regex, but it's not as safe
	#string = string.strip().replace('\\','').replace('\n','').replace(' ','').replace(',','')
	string = re.sub(r'(?i)[^a-z0-9]','', string)
	
	""" Convert from unicode hex to utf-8 """
	#return string.strip().decode('hex').decode('raw_unicode_escape', 'replace').encode('utf-8')
	#return string.strip().decode('hex').encode('utf-8').decode('string_escape')
	return string.strip().decode('hex')

def utf8tohex(string):
	# Convert from utf-8 to unicode hex
	string = string.decode('utf-8').encode('raw_unicode_escape').encode('hex')+'00'
	# Format unicode hex into Wine Registry format (comma separated chars)
	return ','.join( [string[i]+string[i+1] for i in range(0, len(string), 2)] )

def stringtoutf8(string):
	return string.encode('string_escape').encode('utf-8').replace('\\\\', '\\').replace('\\\\', '\\').replace('\x00','')

def wintounix(path):
	if path[1:3] != ':\\' and not path.startswith('..\\') and not path.startswith('.\\'):
		return path
	path = "%s/drive_c/%s" % (os.environ['WINEPREFIX'], "/".join(path[3:].split("\\")).replace('\0',''))
	if not os.path.exists(path):
		newpath = ''
		for branch in filter(len, path.split('/')):
			temppath = '%s/%s' % (newpath, branch)
			if not os.path.exists(temppath):
				pathlist = os.listdir(newpath)
				for filename in pathlist:
					if '~' in branch and filename.lower().startswith(branch.lower().split('~')[0]) \
					or filename.lower() == branch.lower():
						temppath = '%s/%s' % (newpath, filename)
						break
			if os.path.exists(temppath):
				newpath = temppath
			else:
				return path
		path = newpath
	return os.path.normpath(path)

def tempstring():
    return ''.join([random.choice(string.letters+'0123456789') for i in range(5)])

def tempname(prefix='', ext=''):
	return "/tmp/%s%s%s" % (prefix, tempstring(), ext)

def getCommandOutput(command, stderr=False):
	if type(command) not in [list, tuple, set]:
		command = splitString(command)
	else:
		command = list(command)
	out, err = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
	if stderr:
		return out.strip(), err
	else:
		return out.strip()
		
def dictToCaseInsensitive(dict):
	dict_copy = {}
	for key,value in dict.iteritems():
		dict_copy[key.lower()] = value
	return dict_copy

"""
	We define our own string splitting function since the one in shlex can
	crash on badly balanced substrings and doesn't understand substrings in substrings
"""
def splitString(string, remove_escapes=True):
	if remove_escapes:
		return [i.strip('"').strip("'").decode('string_escape') for i in re.split(r'(\s+|(?<!\\)".*?(?<!\\)"|(?<!\\)\'.*?(?<!\\)\')', string) if i.strip()]
	else:
		return [i.strip('"').strip("'") for i in re.split(r'(\s+|(?<!\\)".*?(?<!\\)"|(?<!\\)\'.*?(?<!\\)\')', string) if i.strip()]

def getIconPathFromName(name):
	if name == None or not len(name.strip()):
		return None
	if name.startswith('/') and os.path.exists(name):
		return name
	for path in ['%s/.icons' % getRealHome(), os.path.expanduser('~/.icons'), '%s/.local/share/icons' % getRealHome(), os.path.expanduser('~/.local/share/icons'), '/usr/share/icons/hicolor', '/usr/share/icons', '/usr/share/pixmaps/']:
		if os.path.isfile("%s/%s" % (path, name)):
			return "%s/%s" % (path, name)
		elif os.path.isdir(path):
			for filename in os.listdir(path):
				if filename.startswith(name):
					return "%s/%s" % (path, filename)
	return None

def getRealHome():	
	# Figure out the real home, if we're in a bottle
	if os.path.normpath(os.path.expandvars("$HOME/..")) == "/home":
		return os.path.expandvars("$HOME")
	else:
		return os.path.normpath(os.path.expandvars("$HOME/../.."))

def getUserDefaultShell():
	f = open('/etc/passwd', 'r')
	shell = None
	for i in f.readlines():
		if i.startswith(os.environ['USER']):
			shell = i
			break
	f.close()
	if shell:
		return shell.split(':')[-1].strip()
	else:
		return "/usr/sh"

"""
	The following functions are based on functions in the xdg-utils package
	(http://webcvs.freedesktop.org/portland/portland/xdg-utils/)
"""
def getDesktopEnvironment():
	env =  dict((k.lower(), v) for (k, v) in os.environ.iteritems())
	if 'gnome_desktop_session_id' in env and env['gnome_desktop_session_id'] != "":
		return "gnome"
	elif 'kde_full_session' in env and env['kde_full_session'] == "true":
		return "kde"
	elif subprocess.call("xprop -root _DT_SAVE_MODE | grep ' = \"xfce4\"$'", shell=True) == 0:
		return "xfce"
	else:
		return "unknown"
	
def getDefaultTerminal():
	desktop = getDesktopEnvironment()
	if desktop == "gnome":
		terminal = getCommandOutput('gconftool-2 --get "/desktop/gnome/applications/terminal/exec"')
		terminal = (terminal, [getCommandOutput('gconftool-2 --get "/desktop/gnome/applications/terminal/exec_arg"')])
	elif desktop == "kde":
		terminal = getCommandOutput('kreadconfig --file kdeglobals --group General --key TerminalApplication --default konsole'), ['-e']
	elif desktop == "xfce":
		terminal = 'exo-open', ['--launch', 'TerminalEmulator']
	else:
		terminal = 'xterm', ['-e']
	return terminal

def runInTerminal(shellcommand):
	terminal, args = getDefaultTerminal()
	env = os.environ.copy()
	env['HOME'] = getRealHome()
	return subprocess.Popen([terminal]+args+[shellcommand], env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

def openTerminal(cwd=None):
	terminal, args = getDefaultTerminal()
	shell = getUserDefaultShell()
	if cwd == None or not os.path.isdir(cwd):
		# If no (existing) current work dir was given, choose the first (alphabetically) virtual drive in the Wine configuration 
		cwd = '%s/%s' % ( os.environ['WINEPREFIX'], [ i for i in sorted(os.listdir('/home/dannie/.wine/'), key=str.lower) if i.startswith('drive_') ][0] )
	env = os.environ.copy()
	env['HOME'] = getRealHome()
	return subprocess.Popen([terminal]+args+[shell], cwd=cwd, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
