#!/bin/bash

name="vineyard"
#dir="trunk"
dir="$(basename "$(readlink -f "$(dirname "$0")/../")")"

path="$(readlink -f "$(dirname "$0")/../../")"
version="$(head "$path/$dir/debian/changelog" -n 1 | cut -d\( -f2 | cut -d\) -f1)"
sourceversion="$(echo $version | cut -d- -f1)"

sourcedir="$name"-"$sourceversion"
sourcetar="$name"_"$sourceversion.orig.tar.gz"

cd "$path"

echo -n "Copying the $name directory to /tmp..."
if [ -d "/tmp/$sourcedir" ]; then
	rm -R "/tmp/$sourcedir"
fi
cp -R "$dir" "/tmp/$sourcedir"
rm -R "/tmp/$sourcedir/.bzr" "/tmp/$sourcedir/tools" "/tmp/$sourcedir/python-wine"
echo " done."

echo -n "Compiling MAN page..."
pod2man --section=1 --release=$sourceversion --center="Windows On Unix" "/tmp/$sourcedir/data/vineyard-preferences.pod" > "/tmp/$sourcedir/data/vineyard-preferences.1"
gzip -9 "/tmp/$sourcedir/data/vineyard-preferences.1"
echo " done."

cd /tmp

echo -n "Creating TAR.GZ archive of $name version $sourceversion..."
tar --exclude "$sourcedir/.bzr" --exclude "$sourcedir/.bzrignore" --exclude "$sourcedir/debian" --exclude "$sourcedir/python-wine" --exclude "$sourcedir/tools" --exclude "$sourcedir/data/cache.pyc" -czf "$sourcetar" "$sourcedir"
cp "$sourcetar" "$sourcedir.tar.gz"
echo " done."

echo -n "Signing TAR.GZ archive..."
gpg --armor --sign --detach-sig "$sourcedir.tar.gz"
echo " done."

echo -n "Creating binary package of $name version $version..."
cd "$sourcedir"
debuild -sa
cd ..
echo " done."

echo -n "Creating source package of $name version $version..."
cd "$sourcedir"
debuild -S -sa
cd ..
echo " done."

